"""
Contains ObjectReference class
"""

from typing import Optional

from avionix.kube.base_objects import Core


class ObjectReference(Core):
    """
    :param field_path: If referring to a piece of an object instead of an entire \
        object, this string should contain a valid JSON/Go field access statement, \
        such as desiredState.manifest.containers[2]. For example, if the object \
        reference is to a container within a pod, this would take on a value like: \
        "spec.containers{name}" (where "name" refers to the name of the container that \
        triggered the event) or if no container name is specified "spec.containers[2]" \
        (container with index 2 in this pod). This syntax is chosen only to have some \
        well-defined way of referencing a part of an object.
    :param name: Name of the referent. More info: \
        https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names  # noqa
    :param namespace: Namespace of the referent. More info: \
        https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/
    :param resource_version: Specific resourceVersion to which this reference is made, \
        if any. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#concurrency-control-and-consistency  # noqa
    :param uid: UID of the referent. More info: \
        https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#uids
    :param api_version: API version of the referent.
    """

    def __init__(
        self,
        field_path: str,
        name: Optional[str] = None,
        namespace: Optional[str] = None,
        resource_version: Optional[str] = None,
        uid: Optional[str] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.fieldPath = field_path
        self.name = name
        self.namespace = namespace
        self.resourceVersion = resource_version
        self.uid = uid



from typing import Optional

from avionix._process_utils import custom_check_output
from avionix.kube.base_objects import HelmYaml


class ChartDependency(HelmYaml):
    """
    An object that is equivalent to listing a chart dependency in chart info in helm

    You can also set the values.yaml through this configuration to set any values
    that need to be configured within dependencies

    :param name: Name of chart
    :param version: The version of the chart
    :param repository: The url of the repository that this chart originates from
    :param values: A dictionary representing the yaml to be output in the values.yaml \
        file for this dependency
    """

    def __init__(
        self,
        name: str,
        version: str,
        repository: str,
        local_repo_name: str,
        values: Optional[dict] = None,
    ):
        self.name = name
        self.version = version
        self.repository = repository
        self.__values = values
        self.__local_repo_name = local_repo_name

    def get_values_yaml(self):
        return {self.name: self.__values}

    def add_repo(self):
        custom_check_output(f"helm repo add {self.__local_repo_name} {self.repository}")

    @property
    def local_repo_name(self):
        return self.__local_repo_name



"""
Various Endpoint related classes
"""

from typing import List, Optional

from avionix.kube.base_objects import Discovery
from avionix.kube.core import EndpointPort
from avionix.kube.meta import ObjectMeta
from avionix.kube.reference import ObjectReference
from avionix.yaml.yaml_handling import HelmYaml


class EndpointConditions(HelmYaml):
    """
    :param ready: ready indicates that this endpoint is prepared to receive traffic, \
        according to whatever system is managing the endpoint. A nil value indicates \
        an unknown state. In most cases consumers should interpret this unknown state \
        as ready.
    """

    def __init__(self, ready: bool):
        self.ready = ready


class Endpoint(HelmYaml):
    """
    :param addresses: addresses of this endpoint. The contents of this field are \
        interpreted according to the corresponding EndpointSlice addressType field. \
        Consumers must handle different types of addresses in the context of their own \
        capabilities. This must contain at least one address but no more than 100.
    :param conditions: conditions contains information about the current status of the \
        endpoint.
    :param hostname: hostname of this endpoint. This field may be used by consumers of \
        endpoints to distinguish endpoints from each other (e.g. in DNS names). \
        Multiple endpoints which use the same hostname should be considered fungible \
        (e.g. multiple A values in DNS). Must pass DNS Label (RFC 1123) validation.
    :param target_ref: targetRef is a reference to a Kubernetes object that represents \
        this endpoint.
    :param topology: topology contains arbitrary topology information associated with \
        the endpoint. These key/value pairs must conform with the label format. \
        https://kubernetes.io/docs/concepts/overview/working-with-objects/labels \
        Topology may include a maximum of 16 key/value pairs. This includes, but is \
        not limited to the following well known keys: * kubernetes.io/hostname: the \
        value indicates the hostname of the node   where the endpoint is located. This \
        should match the corresponding   node label. * topology.kubernetes.io/zone: \
        the value indicates the zone where the   endpoint is located. This should \
        match the corresponding node label. * topology.kubernetes.io/region: the value \
        indicates the region where the   endpoint is located. This should match the \
        corresponding node label.
    """

    def __init__(
        self,
        addresses: List[str],
        conditions: Optional[EndpointConditions] = None,
        hostname: Optional[str] = None,
        target_ref: Optional[ObjectReference] = None,
        topology: Optional[dict] = None,
    ):
        self.addresses = addresses
        self.conditions = conditions
        self.hostname = hostname
        self.targetRef = target_ref
        self.topology = topology


class EndpointSlice(Discovery):
    """
    :param metadata: Standard object's metadata.
    :param address_type: addressType specifies the type of address carried by this \
        EndpointSlice. All addresses in this slice must be the same type. This field \
        is immutable after creation. The following address types are currently \
        supported: * IPv4: Represents an IPv4 Address. * IPv6: Represents an IPv6 \
        Address. * FQDN: Represents a Fully Qualified Domain Name.
    :param endpoints: endpoints is a list of unique endpoints in this slice. Each slice \
        may include a maximum of 1000 endpoints.
    :param ports: ports specifies the list of network ports exposed by each endpoint in \
        this slice. Each port must have a unique name. When ports is empty, it \
        indicates that there are no defined ports. When a port is defined with a nil \
        port value, it indicates "all ports". Each slice may include a maximum of 100 \
        ports.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    _non_standard_version = "v1beta1"

    def __init__(
        self,
        metadata: ObjectMeta,
        address_type: str,
        endpoints: List[Endpoint],
        ports: Optional[List[EndpointPort]] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.addressType = address_type
        self.endpoints = endpoints
        self.ports = ports



from avionix.yaml.yaml_handling import HelmYaml


class Values:
    """
    An abstraction on the Helm *values.yaml*.

    More information on *values.yaml* can be found
    `here <https://helm.sh/docs/chart_template_guide/values_files/>`__

    :param values: A dictionary specifying what should appear in *values.yaml*
    """

    def __init__(self, values: dict):
        self.values = values


class Value(HelmYaml):
    """
    Used to reference a value specified in :any:`Values`.

    :param value_reference: A reference string specifying a value in **values.yaml**
        using dot notation.

        For example,

        The string

        .. code-block:: python

            'is.my.value'

        would be translated in the yaml output to

        .. code-block:: python

            '{{ .Values.is.my.value }}'

        which in helm will retrieve the value specified in :any:`Values` at

        .. code-block:: python

            {"is":
                {"my":
                    {
                     "value": <some_value>
                    }
                }
            }
    """

    def __init__(self, value_reference: str):
        self.value_reference = value_reference

    def to_dict(self):
        return "{{" + f" .Values.{self.value_reference} " + "}}"



"""
Contains Lease class
"""

from datetime import time
from typing import Optional

from avionix.kube.base_objects import Coordination
from avionix.kube.meta import ObjectMeta
from avionix.yaml.yaml_handling import HelmYaml


class LeaseSpec(HelmYaml):
    """
    :param acquire_time: acquireTime is a time when the current lease was acquired.
    :param holder_identity: holderIdentity contains the identity of the holder of a \
        current lease.
    :param lease_duration_seconds: leaseDurationSeconds is a duration that candidates \
        for a lease need to wait to force acquire it. This is measure against time of \
        last observed RenewTime.
    :param lease_transitions: leaseTransitions is the number of transitions of a lease \
        between holders.
    :param renew_time: renewTime is a time when the current holder of a lease has last \
        updated the lease.
    """

    def __init__(
        self,
        acquire_time: Optional[time] = None,
        holder_identity: Optional[str] = None,
        lease_duration_seconds: Optional[int] = None,
        lease_transitions: Optional[int] = None,
        renew_time: Optional[time] = None,
    ):
        self.acquireTime = acquire_time
        self.holderIdentity = holder_identity
        self.leaseDurationSeconds = lease_duration_seconds
        self.leaseTransitions = lease_transitions
        self.renewTime = renew_time


class Lease(Coordination):
    """
    :param metadata: More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata  # noqa
    :param spec: Specification of the Lease. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status  # noqa
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self, metadata: ObjectMeta, spec: LeaseSpec, api_version: Optional[str] = None
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec



# flake8: noqa

from avionix.chart.chart_builder import ChartBuilder
from avionix.chart.chart_dependency import ChartDependency
from avionix.chart.chart_info import ChartInfo
from avionix.chart.chart_maintainer import ChartMaintainer
from avionix.chart.values_yaml import Value, Values



"""
Classes related to running and scheduling Kubernetes jobs
"""

from typing import Optional

from avionix.kube.base_objects import Batch
from avionix.kube.core import PodTemplateSpec
from avionix.kube.meta import LabelSelector, ObjectMeta
from avionix.yaml.yaml_handling import HelmYaml


class JobSpec(HelmYaml):
    """
    :param template: Describes the pod that will be created when executing a job. More \
        info: \
        https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/  # noqa
    :param completions: Specifies the desired number of successfully finished pods the \
        job should be run with.  Setting to nil means that the success of any pod \
        signals the success of all pods, and allows parallelism to have any positive \
        value.  Setting to 1 means that parallelism is limited to 1 and the success of \
        that pod signals the success of the job. More info: \
        https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/  # noqa
    :param manual_selector: manualSelector controls generation of pod labels and pod \
        selectors. Leave `manualSelector` unset unless you are certain what you are \
        doing. When false or unset, the system pick labels unique to this job and \
        appends those labels to the pod template.  When true, the user is responsible \
        for picking unique labels and specifying the selector.  Failure to pick a \
        unique label may cause this and other jobs to not function correctly.  \
        However, You may see `manualSelector=true` in jobs that were created with the \
        old `extensions/v1beta1` API. More info: \
        https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/#specifying-your-own-pod-selector  # noqa
    :param parallelism: Specifies the maximum desired number of pods the job should run \
        at any given time. The actual number of pods running in steady state will be \
        less than this number when ((.spec.completions - .status.successful) < \
        .spec.parallelism), i.e. when the work left to do is less than max \
        parallelism. More info: \
        https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/  # noqa
    :param selector: A label query over pods that should match the pod count. Normally, \
        the system sets this field for you. More info: \
        https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors  # noqa
    :param ttl_seconds_after_finished: ttlSecondsAfterFinished limits the lifetime of a \
        Job that has finished execution (either Complete or Failed). If this field is \
        set, ttlSecondsAfterFinished after the Job finishes, it is eligible to be \
        automatically deleted. When the Job is being deleted, its lifecycle guarantees \
        (e.g. finalizers) will be honored. If this field is unset, the Job won't be \
        automatically deleted. If this field is set to zero, the Job becomes eligible \
        to be deleted immediately after it finishes. This field is alpha-level and is \
        only honored by servers that enable the TTLAfterFinished feature.
    :param active_deadline_seconds: Specifies the duration in seconds relative to the \
        startTime that the job may be active before the system tries to terminate it; \
        value must be positive integer
    :param backoff_limit: Specifies the number of retries before marking this job \
        failed. Defaults to 6
    """

    def __init__(
        self,
        template: PodTemplateSpec,
        completions: Optional[int] = None,
        manual_selector: Optional[bool] = None,
        parallelism: Optional[int] = None,
        selector: Optional[LabelSelector] = None,
        ttl_seconds_after_finished: Optional[int] = None,
        active_deadline_seconds: Optional[int] = None,
        backoff_limit: Optional[int] = None,
    ):
        self.template = template
        self.completions = completions
        self.manualSelector = manual_selector
        self.parallelism = parallelism
        self.selector = selector
        self.ttlSecondsAfterFinished = ttl_seconds_after_finished
        self.activeDeadlineSeconds = active_deadline_seconds
        self.backoffLimit = backoff_limit


class Job(Batch):
    """
    :param metadata: Standard object's metadata. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata  # noqa
    :param spec: Specification of the desired behavior of a job. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status  # noqa
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self, metadata: ObjectMeta, spec: JobSpec, api_version: Optional[str] = None
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec


class JobTemplateSpec(HelmYaml):
    """
    :param spec: Specification of the desired behavior of the job. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status  # noqa
    :param metadata: Standard object's metadata of the jobs created from this template. \
        More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata  # noqa
    """

    def __init__(
        self, spec: JobSpec, metadata: Optional[ObjectMeta] = None,
    ):
        self.metadata = metadata
        self.spec = spec


class CronJobSpec(HelmYaml):
    """
    :param job_template: Specifies the job that will be created when executing a \
        CronJob.
    :param schedule: The schedule in Cron format, see \
        https://en.wikipedia.org/wiki/Cron.
    :param starting_deadline_seconds: Optional deadline in seconds for starting the job \
        if it misses scheduled time for any reason.  Missed jobs executions will be \
        counted as failed ones.
    :param concurrency_policy: Specifies how to treat concurrent executions of a Job. \
        Valid values are: - "Allow" (default): allows CronJobs to run concurrently; - \
        "Forbid": forbids concurrent runs, skipping next run if previous run hasn't \
        finished yet; - "Replace": cancels currently running job and replaces it with \
        a new one
    :param failed_jobs_history_limit: The number of failed finished jobs to retain. \
        This is a pointer to distinguish between explicit zero and not specified. \
        Defaults to 1.
    :param successful_jobs_history_limit: The number of successful finished jobs to \
        retain. This is a pointer to distinguish between explicit zero and not \
        specified. Defaults to 3.
    :param suspend: This flag tells the controller to suspend subsequent executions, it \
        does not apply to already started executions.  Defaults to false.
    """

    def __init__(
        self,
        job_template: JobTemplateSpec,
        schedule: str,
        starting_deadline_seconds: Optional[int] = None,
        concurrency_policy: Optional[str] = None,
        failed_jobs_history_limit: Optional[int] = None,
        successful_jobs_history_limit: Optional[int] = None,
        suspend: Optional[bool] = None,
    ):
        self.concurrencyPolicy = concurrency_policy
        self.jobTemplate = job_template
        self.schedule = schedule
        self.startingDeadlineSeconds = starting_deadline_seconds
        self.failedJobsHistoryLimit = failed_jobs_history_limit
        self.successfulJobsHistoryLimit = successful_jobs_history_limit
        self.suspend = suspend


class CronJob(Batch):
    """
    :param metadata: Standard object's metadata. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata  # noqa
    :param spec: Specification of the desired behavior of a cron job, including the \
        schedule. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status  # noqa
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    _non_standard_version = "v1beta1"

    def __init__(
        self, metadata: ObjectMeta, spec: CronJobSpec, api_version: Optional[str] = None
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec



from typing import List, Optional

from avionix.chart.chart_dependency import ChartDependency
from avionix.chart.chart_maintainer import ChartMaintainer
from avionix.kube.base_objects import HelmYaml


class ChartInfo(HelmYaml):
    """
    :param api_version: The chart API version (required)
    :param name: The name of the chart (required)
    :param version: A SemVer 2 version (required)
    :param kube_version: A SemVer range of compatible Kubernetes versions (optional)
    :param description: A single-sentence description of this project (optional)
    :param type: The type of the chart (optional)
    :param key_words: A list of keywords about this project (optional)
    :param home: The URL of this projects home page (optional)
    :param sources: A list of URLs to source code for this project (optional)
    :param dependencies: A list of the chart requirements (optional)
    :param maintainers: A list of maintainers
    :param icon: A URL to an SVG or PNG image to be used as an icon (optional).
    :param app_version: The version of the app that this contains (optional). This \
        needn't be SemVer.
    :param deprecated: Whether this chart is deprecated (optional, boolean)
    :param annotations: A list of annotations keyed by name (optional).
    """

    def __init__(
        self,
        api_version: str,
        name: str,
        version: str,
        kube_version: Optional[str] = None,
        description: Optional[str] = None,
        type: Optional[str] = None,
        key_words: Optional[List[str]] = None,
        home: Optional[str] = None,
        sources: Optional[List[str]] = None,
        dependencies: Optional[List[ChartDependency]] = None,
        maintainers: Optional[List[ChartMaintainer]] = None,
        icon: Optional[str] = None,
        app_version: Optional[str] = None,
        deprecated: Optional[bool] = None,
        annotations: Optional[dict] = None,
    ):
        self.apiVersion = api_version
        self.name = name
        self.version = version
        self.kubeVersion = kube_version
        self.description = description
        self.type = type
        self.keywords = key_words
        self.home = home
        self.sources = sources
        self.dependencies = dependencies if dependencies is not None else []
        self.maintainers = maintainers
        self.icon = icon
        self.appVersion = app_version
        self.deprecated = deprecated
        self.annotations = annotations



"""
Contains various policy based classes
"""

from typing import List, Optional, Union

from avionix.kube.base_objects import Policy
from avionix.kube.core import SELinuxOptions
from avionix.kube.meta import DeleteOptions, LabelSelector, ObjectMeta
from avionix.yaml.yaml_handling import HelmYaml


class IDRange(HelmYaml):
    """
    :param min: min is the start of the range, inclusive.
    :param max: max is the end of the range, inclusive.
    """

    def __init__(self, min: int, max: int):
        self.max = max
        self.min = min


class SupplementalGroupsStrategyOptions(HelmYaml):
    """
    :param rule: rule is the strategy that will dictate what supplemental groups is \
        used in the SecurityContext.
    :param ranges: ranges are the allowed ranges of supplemental groups.  If you would \
        like to force a single supplemental group then supply a single range with the \
        same start and end. Required for MustRunAs.
    """

    def __init__(self, rule: str, ranges: Optional[List[IDRange]] = None):
        self.ranges = ranges
        self.rule = rule


class RuntimeClassStrategyOptions(HelmYaml):
    """
    :param allowed_runtime_class_names: allowedRuntimeClassNames is a whitelist of \
        RuntimeClass names that may be specified on a pod. A value of "*" means that \
        any RuntimeClass name is allowed, and must be the only item in the list. An \
        empty list requires the RuntimeClassName field to be unset.
    :param default_runtime_class_name: defaultRuntimeClassName is the default \
        RuntimeClassName to set on the pod. The default MUST be allowed by the \
        allowedRuntimeClassNames list. A value of nil does not mutate the Pod.
    """

    def __init__(
        self,
        allowed_runtime_class_names: List[str],
        default_runtime_class_name: Optional[str] = None,
    ):
        self.allowedRuntimeClassNames = allowed_runtime_class_names
        self.defaultRuntimeClassName = default_runtime_class_name


class PodDisruptionBudgetSpec(HelmYaml):
    """
    :param max_unavailable: An eviction is allowed if at most "maxUnavailable" pods \
        selected by "selector" are unavailable after the eviction, i.e. even in \
        absence of the evicted pod. For example, one can prevent all voluntary \
        evictions by specifying 0. This is a mutually exclusive setting with \
        "minAvailable".
    :param min_available: An eviction is allowed if at least "minAvailable" pods \
        selected by "selector" will still be available after the eviction, i.e. even \
        in the absence of the evicted pod.  So for example you can prevent all \
        voluntary evictions by specifying "100%".
    :param selector: Label query over pods whose evictions are managed by the \
        disruption budget.
    """

    def __init__(
        self,
        max_unavailable: Optional[Union[int, str]] = None,
        min_available: Optional[Union[int, str]] = None,
        selector: Optional[LabelSelector] = None,
    ):
        self.maxUnavailable = max_unavailable
        self.minAvailable = min_available
        self.selector = selector


class PodDisruptionBudget(Policy):
    """
    :param metadata: None
    :param spec: Specification of the desired behavior of the PodDisruptionBudget.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    _non_standard_version = "v1beta1"

    def __init__(
        self,
        metadata: ObjectMeta,
        spec: PodDisruptionBudgetSpec,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec


class Eviction(Policy):
    """
    :param metadata: ObjectMeta describes the pod that is being evicted.
    :param delete_options: DeleteOptions may be provided
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    _non_standard_version = "v1beta1"

    def __init__(
        self,
        metadata: ObjectMeta,
        delete_options: DeleteOptions,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.deleteOptions = delete_options


class HostPortRange(HelmYaml):
    """
    :param max: max is the end of the range, inclusive.
    :param min: min is the start of the range, inclusive.
    """

    def __init__(self, max: int, min: int):
        self.max = max
        self.min = min


class AllowedHostPath(HelmYaml):
    """
    :param path_prefix: pathPrefix is the path prefix that the host volume must match. \
        It does not support `*`. Trailing slashes are trimmed when validating the path \
        prefix with a host path.  Examples: `/foo` would allow `/foo`, `/foo/` and \
        `/foo/bar` `/foo` would not allow `/food` or `/etc/foo`
    :param read_only: when set to true, will allow host volumes matching the pathPrefix \
        only if all volume mounts are readOnly.
    """

    def __init__(self, path_prefix: str, read_only: Optional[bool] = None):
        self.pathPrefix = path_prefix
        self.readOnly = read_only


class RunAsUserStrategyOptions(HelmYaml):
    """
    :param rule: rule is the strategy that will dictate the allowable RunAsUser values \
        that may be set.
    :param ranges: ranges are the allowed ranges of uids that may be used. If you would \
        like to force a single uid then supply a single range with the same start and \
        end. Required for MustRunAs.
    """

    def __init__(self, rule: str, ranges: Optional[List[IDRange]] = None):
        self.ranges = ranges
        self.rule = rule


class AllowedCSIDriver(HelmYaml):
    """
    :param name: Name is the registered name of the CSI driver
    """

    def __init__(self, name: str):
        self.name = name


class SELinuxStrategyOptions(HelmYaml):
    """
    :param rule: rule is the strategy that will dictate the allowable labels that may \
        be set.
    :param se_linux_options: seLinuxOptions required to run as; required for MustRunAs \
        More info: \
        https://kubernetes.io/docs/tasks/configure-pod-container/security-context/
    """

    def __init__(self, rule: str, se_linux_options: Optional[SELinuxOptions] = None):
        self.rule = rule
        self.seLinuxOptions = se_linux_options


class AllowedFlexVolume(HelmYaml):
    """
    :param driver: driver is the name of the Flexvolume driver.
    """

    def __init__(self, driver: str):
        self.driver = driver


class FSGroupStrategyOptions(HelmYaml):
    """
    :param rule: rule is the strategy that will dictate what FSGroup is used in the \
        SecurityContext.
    :param ranges: ranges are the allowed ranges of fs groups.  If you would like to \
        force a single fs group then supply a single range with the same start and \
        end. Required for MustRunAs.
    """

    def __init__(self, rule: str, ranges: Optional[List[IDRange]] = None):
        self.ranges = ranges
        self.rule = rule


class RunAsGroupStrategyOptions(HelmYaml):
    """
    :param rule: rule is the strategy that will dictate the allowable RunAsGroup values \
        that may be set.
    :param ranges: ranges are the allowed ranges of gids that may be used. If you would \
        like to force a single gid then supply a single range with the same start and \
        end. Required for MustRunAs.
    """

    def __init__(self, rule: str, ranges: Optional[List[IDRange]] = None):
        self.ranges = ranges
        self.rule = rule


class PodSecurityPolicySpec(HelmYaml):
    """
    :param fs_group: fsGroup is the strategy that will dictate what fs group is used by \
        the SecurityContext.
    :param run_as_user: runAsUser is the strategy that will dictate the allowable \
        RunAsUser values that may be set.
    :param se_linux: seLinux is the strategy that will dictate the allowable labels \
        that may be set.
    :param allowed_csidrivers: AllowedCSIDrivers is a whitelist of inline CSI drivers \
        that must be explicitly set to be embedded within a pod spec. An empty value \
        indicates that any CSI driver can be used for inline ephemeral volumes. This \
        is an alpha field, and is only honored if the API server enables the \
        CSIInlineVolume feature gate.
    :param allowed_capabilities: allowedCapabilities is a list of capabilities that can \
        be requested to add to the container. Capabilities in this field may be added \
        at the pod author's discretion. You must not list a capability in both \
        allowedCapabilities and requiredDropCapabilities.
    :param allowed_flex_volumes: allowedFlexVolumes is a whitelist of allowed \
        Flexvolumes.  Empty or nil indicates that all Flexvolumes may be used.  This \
        parameter is effective only when the usage of the Flexvolumes is allowed in \
        the "volumes" field.
    :param allowed_host_paths: allowedHostPaths is a white list of allowed host paths. \
        Empty indicates that all host paths may be used.
    :param allowed_proc_mount_types: AllowedProcMountTypes is a whitelist of allowed \
        ProcMountTypes. Empty or nil indicates that only the DefaultProcMountType may \
        be used. This requires the ProcMountType feature flag to be enabled.
    :param default_add_capabilities: defaultAddCapabilities is the default set of \
        capabilities that will be added to the container unless the pod spec \
        specifically drops the capability.  You may not list a capability in both \
        defaultAddCapabilities and requiredDropCapabilities. Capabilities added here \
        are implicitly allowed, and need not be included in the allowedCapabilities \
        list.
    :param default_allow_privilege_escalation: defaultAllowPrivilegeEscalation controls \
        the default setting for whether a process can gain more privileges than its \
        parent process.
    :param host_ipc: hostIPC determines if the policy allows the use of HostIPC in the \
        pod spec.
    :param host_pid: hostPID determines if the policy allows the use of HostPID in the \
        pod spec.
    :param host_ports: hostPorts determines which host port ranges are allowed to be \
        exposed.
    :param privileged: privileged determines if a pod can request to be run as \
        privileged.
    :param read_only_root_filesystem: readOnlyRootFilesystem when set to true will \
        force containers to run with a read only root file system.  If the container \
        specifically requests to run with a non-read only root file system the PSP \
        should deny the pod. If set to false the container may run with a read only \
        root file system if it wishes but it will not be forced to.
    :param required_drop_capabilities: requiredDropCapabilities are the capabilities \
        that will be dropped from the container.  These are required to be dropped and \
        cannot be added.
    :param run_as_group: RunAsGroup is the strategy that will dictate the allowable \
        RunAsGroup values that may be set. If this field is omitted, the pod's \
        RunAsGroup can take any value. This field requires the RunAsGroup feature gate \
        to be enabled.
    :param runtime_class: runtimeClass is the strategy that will dictate the allowable \
        RuntimeClasses for a pod. If this field is omitted, the pod's runtimeClassName \
        field is unrestricted. Enforcement of this field depends on the RuntimeClass \
        feature gate being enabled.
    :param supplemental_groups: supplementalGroups is the strategy that will dictate \
        what supplemental groups are used by the SecurityContext.
    :param allow_privilege_escalation: allowPrivilegeEscalation determines if a pod can \
        request to allow privilege escalation. If unspecified, defaults to true.
    :param allowed_unsafe_sysctls: allowedUnsafeSysctls is a list of explicitly allowed \
        unsafe sysctls, defaults to none. Each entry is either a plain sysctl name or \
        ends in "*" in which case it is considered as a prefix of allowed sysctls. \
        Single * means all unsafe sysctls are allowed. Kubelet has to whitelist all \
        allowed unsafe sysctls explicitly to avoid rejection.  Examples: e.g. "foo/*" \
        allows "foo/bar", "foo/baz", etc. e.g. "foo.*" allows "foo.bar", "foo.baz", \
        etc.
    :param forbidden_sysctls: forbiddenSysctls is a list of explicitly forbidden \
        sysctls, defaults to none. Each entry is either a plain sysctl name or ends in \
        "*" in which case it is considered as a prefix of forbidden sysctls. Single * \
        means all sysctls are forbidden.  Examples: e.g. "foo/*" forbids "foo/bar", \
        "foo/baz", etc. e.g. "foo.*" forbids "foo.bar", "foo.baz", etc.
    :param host_network: hostNetwork determines if the policy allows the use of \
        HostNetwork in the pod spec.
    :param volumes: volumes is a white list of allowed volume plugins. Empty indicates \
        that no volumes may be used. To allow all volumes you may use '\\*'.
    """

    def __init__(
        self,
        fs_group: FSGroupStrategyOptions,
        run_as_user: RunAsUserStrategyOptions,
        se_linux: SELinuxStrategyOptions,
        supplemental_groups: SupplementalGroupsStrategyOptions,
        allowed_csidrivers: Optional[List[AllowedCSIDriver]] = None,
        allowed_capabilities: Optional[List[str]] = None,
        allowed_flex_volumes: Optional[List[AllowedFlexVolume]] = None,
        allowed_host_paths: Optional[List[AllowedHostPath]] = None,
        allowed_proc_mount_types: Optional[List[str]] = None,
        default_add_capabilities: Optional[List[str]] = None,
        default_allow_privilege_escalation: Optional[bool] = None,
        host_ipc: Optional[bool] = None,
        host_pid: Optional[bool] = None,
        host_ports: Optional[List[HostPortRange]] = None,
        privileged: Optional[bool] = None,
        read_only_root_filesystem: Optional[bool] = None,
        required_drop_capabilities: Optional[List[str]] = None,
        run_as_group: Optional[RunAsGroupStrategyOptions] = None,
        runtime_class: Optional[RuntimeClassStrategyOptions] = None,
        allow_privilege_escalation: Optional[bool] = None,
        allowed_unsafe_sysctls: Optional[List[str]] = None,
        forbidden_sysctls: Optional[List[str]] = None,
        host_network: Optional[bool] = None,
        volumes: Optional[List[str]] = None,
    ):
        self.allowedCSIDrivers = allowed_csidrivers
        self.allowedCapabilities = allowed_capabilities
        self.allowedFlexVolumes = allowed_flex_volumes
        self.allowedHostPaths = allowed_host_paths
        self.allowedProcMountTypes = allowed_proc_mount_types
        self.defaultAddCapabilities = default_add_capabilities
        self.defaultAllowPrivilegeEscalation = default_allow_privilege_escalation
        self.fsGroup = fs_group
        self.hostIPC = host_ipc
        self.hostPID = host_pid
        self.hostPorts = host_ports
        self.privileged = privileged
        self.readOnlyRootFilesystem = read_only_root_filesystem
        self.requiredDropCapabilities = required_drop_capabilities
        self.runAsGroup = run_as_group
        self.runAsUser = run_as_user
        self.runtimeClass = runtime_class
        self.seLinux = se_linux
        self.supplementalGroups = supplemental_groups
        self.allowPrivilegeEscalation = allow_privilege_escalation
        self.allowedUnsafeSysctls = allowed_unsafe_sysctls
        self.forbiddenSysctls = forbidden_sysctls
        self.hostNetwork = host_network
        self.volumes = volumes


class PodSecurityPolicy(Policy):
    """
    :param metadata: Standard object's metadata. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata  # noqa
    :param spec: spec defines the policy enforced.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    _non_standard_version = "v1beta1"

    def __init__(
        self,
        metadata: ObjectMeta,
        spec: PodSecurityPolicySpec,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec



"""
Classes for creating custom Kubernetes objects
"""

from typing import List, Optional, Union

from avionix.kube.apiregistration import ServiceReference
from avionix.kube.base_objects import ApiExtensions
from avionix.kube.meta import ObjectMeta
from avionix.yaml.yaml_handling import HelmYaml


class JSONSchemaPropsOrBool(HelmYaml):
    """"""

    pass


class WebhookClientConfig(HelmYaml):
    """
    :param ca_bundle: caBundle is a PEM encoded CA bundle which will be used to \
        validate the webhook's server certificate. If unspecified, system trust roots \
        on the apiserver are used.
    :param service: service is a reference to the service for this webhook. Either \
        service or url must be specified.  If the webhook is running within the \
        cluster, then you should use `service`.
    :param url: url gives the location of the webhook, in standard URL form \
        (`scheme://host:port/path`). Exactly one of `url` or `service` must be \
        specified.  The `host` should not refer to a service running in the cluster; \
        use the `service` field instead. The host might be resolved via external DNS \
        in some apiservers (e.g., `kube-apiserver` cannot resolve in-cluster DNS as \
        that would be a layering violation). `host` may also be an IP address.  Please \
        note that using `localhost` or `127.0.0.1` as a `host` is risky unless you \
        take great care to run this webhook on all hosts which run an apiserver which \
        might need to make calls to this webhook. Such installs are likely to be \
        non-portable, i.e., not easy to turn up in a new cluster.  The scheme must be \
        "https"; the URL must begin with "https://".  A path is optional, and if \
        present may be any string permissible in a URL. You may use the path to pass \
        an arbitrary string to the webhook, for example, a cluster identifier.  \
        Attempting to use a user or basic auth e.g. "user:password@" is not allowed. \
        Fragments ("#...") and query parameters ("?...") are not allowed, either.
    """

    def __init__(
        self,
        ca_bundle: Optional[str] = None,
        service: Optional[ServiceReference] = None,
        url: Optional[str] = None,
    ):
        self.caBundle = ca_bundle
        self.service = service
        self.url = url


class JSON(HelmYaml):
    """"""

    pass


class JSONSchemaPropsOrArray(HelmYaml):
    """"""

    pass


class ExternalDocumentation(HelmYaml):
    """
    :param url: None
    :param description: None
    """

    def __init__(self, url: str, description: Optional[str] = None):
        self.url = url
        self.description = description


class JSONSchemaProps(HelmYaml):
    """
    :param type: None
    :param additional_items: None
    :param additional_properties: None
    :param all_of: None
    :param any_of: None
    :param default: default is a default value for undefined object fields. Defaulting \
        is a beta feature under the CustomResourceDefaulting feature gate. Defaulting \
        requires spec.preserveUnknownFields to be false.
    :param definitions: None
    :param dependencies: None
    :param description: None
    :param enum: None
    :param example: None
    :param exclusive_maximum: None
    :param exclusive_minimum: None
    :param external_docs: None
    :param format: format is an OpenAPI v3 format string. Unknown formats are ignored. \
        The following formats are validated:  - bsonobjectid: a bson object ID, i.e. a \
        24 characters hex string - uri: an URI as parsed by Golang \
        net/url.ParseRequestURI - email: an email address as parsed by Golang \
        net/mail.ParseAddress - hostname: a valid representation for an Internet host \
        name, as defined by RFC 1034, section 3.1 [RFC1034]. - ipv4: an IPv4 IP as \
        parsed by Golang net.ParseIP - ipv6: an IPv6 IP as parsed by Golang \
        net.ParseIP - cidr: a CIDR as parsed by Golang net.ParseCIDR - mac: a MAC \
        address as parsed by Golang net.ParseMAC - uuid: an UUID that allows uppercase \
        defined by the regex \
        (?i)^[0-9a-f]{8}-?[0-9a-f]{4}-?[0-9a-f]{4}-?[0-9a-f]{4}-?[0-9a-f]{12}$ - \
        uuid3: an UUID3 that allows uppercase defined by the regex \
        (?i)^[0-9a-f]{8}-?[0-9a-f]{4}-?3[0-9a-f]{3}-?[0-9a-f]{4}-?[0-9a-f]{12}$ - \
        uuid4: an UUID4 that allows uppercase defined by the regex \
        (?i)^[0-9a-f]{8}-?[0-9a-f]{4}-?4[0-9a-f]{3}-?[89ab][0-9a-f]{3}-?[0-9a-f]{12}$ \
        - uuid5: an UUID5 that allows uppercase defined by the regex \
        (?i)^[0-9a-f]{8}-?[0-9a-f]{4}-?5[0-9a-f]{3}-?[89ab][0-9a-f]{3}-?[0-9a-f]{12}$ \
        - isbn: an ISBN10 or ISBN13 number string like "0321751043" or \
        "978-0321751041" - isbn10: an ISBN10 number string like "0321751043" - isbn13: \
        an ISBN13 number string like "978-0321751041" - creditcard: a credit card \
        number defined by the regex \
        ^(?:4[0-9]{12}(?:[0-9]{3})?|5[1-5][0-9]{14}|6(?:011|5[0-9][0-9])[0-9]{12}|3[47][0-9]{13}|3(?:0[0-5]|[68][0-9])[0-9]{11}|(?:2131|1800|35\d{3})\d{11})$  # noqa \
        with any non digit characters mixed in - ssn: a U.S. social security number \
        following the regex ^\d{3}[- ]?\d{2}[- ]?\d{4}$ - hexcolor: an hexadecimal \
        color code like "#FFFFFF: following the regex \
        ^#?([0-9a-fA-F]{3}|[0-9a-fA-F]{6})$ - rgbcolor: an RGB color code like rgb \
        like "rgb(255,255,2559" - byte: base64 encoded binary data - password: any \
        kind of string - date: a date string like "2006-01-02" as defined by full-date \
        in RFC3339 - duration: a duration string like "22 ns" as parsed by Golang \
        time.ParseDuration or compatible with Scala duration format - datetime: a date \
        time string like "2014-12-15T19:30:20.000Z" as defined by date-time in \
        RFC3339.
    :param id: None
    :param items: None
    :param max_items: None
    :param max_length: None
    :param max_properties: None
    :param maximum: None
    :param min_items: None
    :param min_length: None
    :param min_properties: None
    :param minimum: None
    :param multiple_of: None
    :param not_: None
    :param nullable: None
    :param one_of: None
    :param pattern: None
    :param pattern_properties: None
    :param properties: None
    :param required: None
    :param title: None
    :param unique_items: None
    :param x_kubernetes_embedded_resource: x-kubernetes-embedded-resource defines that \
        the value is an embedded Kubernetes runtime.Object, with TypeMeta and \
        ObjectMeta. The type must be object. It is allowed to further restrict the \
        embedded object. kind, apiVersion and metadata are validated automatically. \
        x-kubernetes-preserve-unknown-fields is allowed to be true, but does not have \
        to be if the object is fully specified (up to kind, apiVersion, metadata).
    :param x_kubernetes_int_or_string: x-kubernetes-int-or-string specifies that this \
        value is either an integer or a string. If this is true, an empty type is \
        allowed and type as child of anyOf is permitted if following one of the \
        following patterns:  1) anyOf:    - type: integer    - type: string 2) allOf:  \
          - anyOf:      - type: integer      - type: string    - ... zero or more
    :param x_kubernetes_list_map_keys: x-kubernetes-list-map-keys annotates an array \
        with the x-kubernetes-list-type `map` by specifying the keys used as the index \
        of the map.  This tag MUST only be used on lists that have the \
        "x-kubernetes-list-type" extension set to "map". Also, the values specified \
        for this attribute must be a scalar typed field of the child structure (no \
        nesting is supported).  The properties specified must either be required or \
        have a default value, to ensure those properties are present for all list \
        items.
    :param x_kubernetes_list_type: x-kubernetes-list-type annotates an array to further \
        describe its topology. This extension must only be used on lists and may have \
        3 possible values:
            1) `atomic`: the list is treated as a single entity, like a scalar.
            Atomic lists will be entirely replaced when updated. This extension
            may be used on any type of list (struct, scalar, ...).
            2) `set`: Sets are lists that must not have multiple items with the same \
            value. Each value must be a scalar, an object with x-kubernetes-map-type \
            `atomic` or an array with x-kubernetes-list-type `atomic`.
            3) `map`: These lists are like maps in that their elements have a non-index
            key used to identify them. Order is preserved upon merge. The map tag must \
            only be used on a list with elements of type object. Defaults to atomic
            for arrays.
    :param x_kubernetes_map_type: x-kubernetes-map-type annotates an object to further \
        describe its topology. This extension must only be used when type is object \
        and may have 2 possible values:  1) `granular`:      These maps are actual \
        maps (key-value pairs) and each fields are independent      from each other \
        (they can each be manipulated by separate actors). This is      the default \
        behaviour for all maps. 2) `atomic`: the list is treated as a single entity, \
        like a scalar.      Atomic maps will be entirely replaced when updated.
    :param x_kubernetes_preserve_unknown_fields: x-kubernetes-preserve-unknown-fields \
        stops the API server decoding step from pruning fields which are not specified \
        in the validation schema. This affects fields recursively, but switches back \
        to normal pruning behaviour if nested properties or additionalProperties are \
        specified in the schema. This can either be true or undefined. False is \
        forbidden.
    """

    def __init__(
        self,
        type: Optional[str] = None,
        additional_items: Optional[JSONSchemaPropsOrBool] = None,
        additional_properties: Optional[JSONSchemaPropsOrBool] = None,
        all_of: Optional[List["JSONSchemaProps"]] = None,
        any_of: Optional[List["JSONSchemaProps"]] = None,
        default: Optional[JSON] = None,
        definitions: Optional[dict] = None,
        dependencies: Optional[dict] = None,
        description: Optional[str] = None,
        enum: Optional[List[JSON]] = None,
        example: Optional[JSON] = None,
        exclusive_maximum: Optional[bool] = None,
        exclusive_minimum: Optional[bool] = None,
        external_docs: Optional[ExternalDocumentation] = None,
        format: Optional[str] = None,
        id: Optional[str] = None,
        items: Optional[JSONSchemaPropsOrArray] = None,
        max_items: Optional[int] = None,
        max_length: Optional[int] = None,
        max_properties: Optional[int] = None,
        maximum: Optional[Union[int, float]] = None,
        min_items: Optional[int] = None,
        min_length: Optional[int] = None,
        min_properties: Optional[int] = None,
        minimum: Optional[Union[int, float]] = None,
        multiple_of: Optional[Union[int, float]] = None,
        not_: Optional["JSONSchemaProps"] = None,
        nullable: Optional[bool] = None,
        one_of: Optional[List["JSONSchemaProps"]] = None,
        pattern: Optional[str] = None,
        pattern_properties: Optional[dict] = None,
        properties: Optional[dict] = None,
        required: Optional[List[str]] = None,
        title: Optional[str] = None,
        unique_items: Optional[bool] = None,
        x_kubernetes_embedded_resource: Optional[bool] = None,
        x_kubernetes_int_or_string: Optional[bool] = None,
        x_kubernetes_list_map_keys: Optional[List[str]] = None,
        x_kubernetes_list_type: Optional[str] = None,
        x_kubernetes_map_type: Optional[str] = None,
        x_kubernetes_preserve_unknown_fields: Optional[bool] = None,
    ):
        self.type = type
        self.additionalItems = additional_items
        self.additionalProperties = additional_properties
        self.allOf = all_of
        self.anyOf = any_of
        self.default = default
        self.definitions = definitions
        self.dependencies = dependencies
        self.description = description
        self.enum = enum
        self.example = example
        self.exclusiveMaximum = exclusive_maximum
        self.exclusiveMinimum = exclusive_minimum
        self.externalDocs = external_docs
        self.format = format
        self.id = id
        self.items = items
        self.maxItems = max_items
        self.maxLength = max_length
        self.maxProperties = max_properties
        self.maximum = maximum
        self.minItems = min_items
        self.minLength = min_length
        self.minProperties = min_properties
        self.minimum = minimum
        self.multipleOf = multiple_of
        self["not"] = not_
        self.nullable = nullable
        self.oneOf = one_of
        self.pattern = pattern
        self.patternProperties = pattern_properties
        self.properties = properties
        self.required = required
        self.title = title
        self.uniqueItems = unique_items
        self["x-kubernetes-embedded-resource"] = x_kubernetes_embedded_resource
        self["x-kubernetes-int-or-string"] = x_kubernetes_int_or_string
        self["x-kubernetes-list-map-keys"] = x_kubernetes_list_map_keys
        self["x-kubernetes-list-type"] = x_kubernetes_list_type
        self["x-kubernetes-map-type"] = x_kubernetes_map_type
        self[
            "x-kubernetes-preserve-unknown-fields"
        ] = x_kubernetes_preserve_unknown_fields


class CustomResourceValidation(HelmYaml):
    """
    :param open_apiv3_schema: openAPIV3Schema is the OpenAPI v3 schema to use for \
        validation and pruning.
    """

    def __init__(self, open_apiv3_schema: JSONSchemaProps):
        self.openAPIV3Schema = open_apiv3_schema


class CustomResourceColumnDefinition(HelmYaml):
    """
    :param name: name is a human readable name for the column.
    :param json_path: jsonPath is a simple JSON path (i.e. with array notation) which \
        is evaluated against each custom resource to produce the value for this \
        column.
    :param type: type is an OpenAPI type definition for this column. \
        https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#data-types \
        for details.
    :param description: description is a human readable description of this column.
    :param format: format is an optional OpenAPI type definition for this column. The \
        'name' format is applied to the primary identifier column to assist in clients \
        identifying column is the resource name. See \
        https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#data-types \
        for details.
    :param priority: priority is an integer defining the relative importance of this \
        column compared to others. Lower numbers are considered higher priority. \
        Columns that may be omitted in limited space scenarios should be given a \
        priority greater than 0.
    """

    def __init__(
        self,
        name: str,
        json_path: str,
        type: str,
        description: Optional[str] = None,
        format: Optional[str] = None,
        priority: Optional[int] = None,
    ):
        self.name = name
        self.jsonPath = json_path
        self.type = type
        self.description = description
        self.format = format
        self.priority = priority


class CustomResourceSubresourceScale(HelmYaml):
    """
    :param spec_replicas_path: specReplicasPath defines the JSON path inside of a \
        custom resource that corresponds to Scale `spec.replicas`. Only JSON paths \
        without the array notation are allowed. Must be a JSON Path under `.spec`. If \
        there is no value under the given path in the custom resource, the `/scale` \
        subresource will return an error on GET.
    :param status_replicas_path: statusReplicasPath defines the JSON path inside of a \
        custom resource that corresponds to Scale `status.replicas`. Only JSON paths \
        without the array notation are allowed. Must be a JSON Path under `.status`. \
        If there is no value under the given path in the custom resource, the \
        `status.replicas` value in the `/scale` subresource will default to 0.
    :param label_selector_path: labelSelectorPath defines the JSON path inside of a \
        custom resource that corresponds to Scale `status.selector`. Only JSON paths \
        without the array notation are allowed. Must be a JSON Path under `.status` or \
        `.spec`. Must be set to work with HorizontalPodAutoscaler. The field pointed \
        by this JSON path must be a string field (not a complex selector struct) which \
        contains a serialized label selector in string form. More info: \
        https://kubernetes.io/docs/tasks/access-kubernetes-api/custom-resources/custom-resource-definitions#scale-subresource  # noqa \
        If there is no value under the given path in the custom resource, the \
        `status.selector` value in the `/scale` subresource will default to the empty \
        string.
    """

    def __init__(
        self,
        spec_replicas_path: str,
        status_replicas_path: str,
        label_selector_path: Optional[str] = None,
    ):
        self.labelSelectorPath = label_selector_path
        self.specReplicasPath = spec_replicas_path
        self.statusReplicasPath = status_replicas_path


class CustomResourceSubresources(HelmYaml):
    """
    :param scale: scale indicates the custom resource should serve a `/scale` \
        subresource that returns an `autoscaling/v1` Scale object.
    """

    def __init__(self, scale: CustomResourceSubresourceScale):
        self.scale = scale


class CustomResourceDefinitionVersion(HelmYaml):
    """
    :param name: name is the version name, e.g. “v1”, “v2beta1”, etc. The custom \
        resources are served under this version at `/apis/<group>/<version>/...` if \
        `served` is true.
    :param additional_printer_columns: additionalPrinterColumns specifies additional \
        columns returned in Table output. See \
        https://kubernetes.io/docs/reference/using-api/api-concepts/#receiving-resources-as-tables  # noqa \
        for details. If no columns are specified, a single column displaying the age \
        of the custom resource is used.
    :param schema: schema describes the schema used for validation, pruning, and \
        defaulting of this version of the custom resource.
    :param served: served is a flag enabling/disabling this version from being served \
        via REST APIs
    :param storage: storage indicates this version should be used when persisting \
        custom resources to storage. There must be exactly one version with \
        storage=true.
    :param subresources: subresources specify what subresources this version of the \
        defined custom resource have.
    """

    def __init__(
        self,
        name: str,
        additional_printer_columns: List[CustomResourceColumnDefinition],
        schema: CustomResourceValidation,
        served: bool,
        storage: bool,
        subresources: Optional[CustomResourceSubresources] = None,
    ):
        self.name = name
        self.additionalPrinterColumns = additional_printer_columns
        self.schema = schema
        self.served = served
        self.storage = storage
        self.subresources = subresources


class WebhookConversion(HelmYaml):
    """
    :param client_config: clientConfig is the instructions for how to call the webhook \
        if strategy is `Webhook`.
    :param conversion_review_versions: conversionReviewVersions is an ordered list of \
        preferred `ConversionReview` versions the Webhook expects. The API server will \
        use the first version in the list which it supports. If none of the versions \
        specified in this list are supported by API server, conversion will fail for \
        the custom resource. If a persisted Webhook configuration specifies allowed \
        versions and does not include any versions known to the API Server, calls to \
        the webhook will fail.
    """

    def __init__(
        self, client_config: WebhookClientConfig, conversion_review_versions: List[str]
    ):
        self.clientConfig = client_config
        self.conversionReviewVersions = conversion_review_versions


class CustomResourceConversion(HelmYaml):
    """
    :param webhook: webhook describes how to call the conversion webhook. Required when \
        `strategy` is set to `Webhook`.
    :param strategy: strategy specifies how custom resources are converted between \
        versions. Allowed values are:
            - `None`: The converter only change the apiVersion and would not touch any
                other field in the custom resource.
            - `Webhook`: API Server will call to an external webhook to do the
                conversion.

        Additional information is needed for this option. This requires \
        spec.preserveUnknownFields to be false, and spec.conversion.webhook to be set.
    """

    def __init__(self, webhook: WebhookConversion, strategy: str):
        self.webhook = webhook
        self.strategy = strategy


class CustomResourceDefinitionNames(HelmYaml):
    """
    :param categories: categories is a list of grouped resources this custom resource \
        belongs to (e.g. 'all'). This is published in API discovery documents, and \
        used by clients to support invocations like `kubectl get all`.
    :param kind: kind is the serialized kind of the resource. It is normally CamelCase \
        and singular. Custom resource instances will use this value as the `kind` \
        attribute in API calls.
    :param plural: plural is the plural name of the resource to serve. The custom \
        resources are served under `/apis/<group>/<version>/.../<plural>`. Must match \
        the name of the CustomResourceDefinition (in the form \
        `<names.plural>.<group>`). Must be all lowercase.
    :param list_kind: listKind is the serialized kind of the list for this resource. \
        Defaults to `kind` List".
    :param short_names: shortNames are short names for the resource, exposed in API \
        discovery documents, and used by clients to support invocations like `kubectl \
        get <shortname>`. It must be all lowercase.
    :param singular: singular is the singular name of the resource. It must be all \
        lowercase. Defaults to lowercased `kind`.
    """

    def __init__(
        self,
        categories: List[str],
        kind: str,
        plural: str,
        list_kind: Optional[str] = None,
        short_names: Optional[List[str]] = None,
        singular: Optional[str] = None,
    ):
        self.categories = categories
        self.kind = kind
        self.plural = plural
        self.listKind = list_kind
        self.shortNames = short_names
        self.singular = singular


class CustomResourceDefinitionSpec(HelmYaml):
    """
    :param group: group is the API group of the defined custom resource. The custom \
        resources are served under `/apis/<group>/...`. Must match the name of the \
        CustomResourceDefinition (in the form `<names.plural>.<group>`).
    :param names: names specify the resource and kind names for the custom resource.
    :param scope: scope indicates whether the defined custom resource is cluster- or \
        namespace-scoped. Allowed values are `Cluster` and `Namespaced`.
    :param versions: versions is the list of all API versions of the defined custom \
        resource. Version names are used to compute the order in which served versions \
        are listed in API discovery. If the version string is "kube-like", it will \
        sort above non "kube-like" version strings, which are ordered \
        lexicographically. "Kube-like" versions start with a "v", then are followed by \
        a number (the major version), then optionally the string "alpha" or "beta" and \
        another number (the minor version). These are sorted first by GA > beta > \
        alpha (where GA is a version with no suffix such as beta or alpha), and then \
        by comparing major version, then minor version. An example sorted list of \
        versions: v10, v2, v1, v11beta2, v10beta3, v3beta1, v12alpha1, v11alpha2, \
        foo1, foo10.
    :param conversion: conversion defines conversion settings for the CRD.
    :param preserve_unknown_fields: preserveUnknownFields indicates that object fields \
        which are not specified in the OpenAPI schema should be preserved when \
        persisting to storage. apiVersion, kind, metadata and known fields inside \
        metadata are always preserved. This field is deprecated in favor of setting \
        `x-preserve-unknown-fields` to true in \
        `spec.versions[*].schema.openAPIV3Schema`. See \
        https://kubernetes.io/docs/tasks/access-kubernetes-api/custom-resources/custom-resource-definitions/#pruning-versus-preserving-unknown-fields  # noqa \
        for details.
    """

    def __init__(
        self,
        group: str,
        names: CustomResourceDefinitionNames,
        scope: str,
        versions: List[CustomResourceDefinitionVersion],
        conversion: Optional[CustomResourceConversion] = None,
        preserve_unknown_fields: Optional[bool] = None,
    ):
        self.group = group
        self.names = names
        self.scope = scope
        self.versions = versions
        self.conversion = conversion
        self.preserveUnknownFields = preserve_unknown_fields


class CustomResourceDefinition(ApiExtensions):
    """
    :param metadata: None
    :param spec: spec describes how the user wants the resources to appear
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ObjectMeta,
        spec: CustomResourceDefinitionSpec,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec



"""
Contains CertificateSigningRequest class
"""

from typing import List, Optional

from avionix.kube.base_objects import Certificates
from avionix.kube.meta import ObjectMeta
from avionix.yaml.yaml_handling import HelmYaml


class CertificateSigningRequestSpec(HelmYaml):
    """
    :param request: Base64-encoded PKCS#10 CSR data
    :param signer_name: Requested signer for the request. It is a qualified name in the \
        form: `scope-hostname.io/name`. If empty, it will be defaulted:  1. If it's a \
        kubelet client certificate, it is assigned     \
        "kubernetes.io/kube-apiserver-client-kubelet".  2. If it's a kubelet serving \
        certificate, it is assigned     "kubernetes.io/kubelet-serving".  3. \
        Otherwise, it is assigned "kubernetes.io/legacy-unknown". Distribution of \
        trust for signers happens out of band. You can select on this field using \
        `spec.signerName`.
    :param usages: allowedUsages specifies a set of usage contexts the key will be \
        valid for. See: https://tools.ietf.org/html/rfc5280#section-4.2.1.3      \
        https://tools.ietf.org/html/rfc5280#section-4.2.1.12
    """

    def __init__(
        self,
        request: str,
        signer_name: Optional[str] = None,
        usages: Optional[List[str]] = None,
    ):
        self.request = request
        self.signerName = signer_name
        self.usages = usages


class CertificateSigningRequest(Certificates):
    """
    :param metadata: None
    :param spec: The certificate request itself and any additional information.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    _non_standard_version = "v1beta1"

    def __init__(
        self,
        metadata: ObjectMeta,
        spec: CertificateSigningRequestSpec,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec



"""
Contains APIService class
"""

from typing import Optional

from avionix.kube.base_objects import ApiRegistration
from avionix.kube.meta import ObjectMeta
from avionix.yaml.yaml_handling import HelmYaml


class ServiceReference(HelmYaml):
    """
    :param name: Name is the name of the service
    :param namespace: Namespace is the namespace of the service
    :param port: If specified, the port on the service that hosting webhook. Default to \
        443 for backward compatibility. `port` should be a valid port number (1-65535, \
        inclusive).
    """

    def __init__(self, name: str, namespace: str, port: Optional[int] = None):
        self.name = name
        self.namespace = namespace
        self.port = port


class APIServiceSpec(HelmYaml):
    """
    :param group: Group is the API group name this server hosts
    :param group_priority_minimum: GroupPriorityMininum is the priority this group \
        should have at least. Higher priority means that the group is preferred by \
        clients over lower priority ones. Note that other versions of this group might \
        specify even higher GroupPriorityMininum values such that the whole group gets \
        a higher priority. The primary sort is based on GroupPriorityMinimum, ordered \
        highest number to lowest (20 before 10). The secondary sort is based on the \
        alphabetical comparison of the name of the object.  (v1.bar before v1.foo) \
        We'd recommend something like: \*.k8s.io (except extensions) at 18000 and \
        PaaSes (OpenShift, Deis) are recommended to be in the 2000s
    :param version: Version is the API version this server hosts.  For example, "v1"
    :param version_priority: VersionPriority controls the ordering of this API version \
        inside of its group.  Must be greater than zero. The primary sort is based on \
        VersionPriority, ordered highest to lowest (20 before 10). Since it's inside \
        of a group, the number can be small, probably in the 10s. In case of equal \
        version priorities, the version string will be used to compute the order \
        inside a group. If the version string is "kube-like", it will sort above non \
        "kube-like" version strings, which are ordered lexicographically. "Kube-like" \
        versions start with a "v", then are followed by a number (the major version), \
        then optionally the string "alpha" or "beta" and another number (the minor \
        version). These are sorted first by GA > beta > alpha (where GA is a version \
        with no suffix such as beta or alpha), and then by comparing major version, \
        then minor version. An example sorted list of versions: v10, v2, v1, v11beta2, \
        v10beta3, v3beta1, v12alpha1, v11alpha2, foo1, foo10.
    :param ca_bundle: CABundle is a PEM encoded CA bundle which will be used to \
        validate an API server's serving certificate. If unspecified, system trust \
        roots on the apiserver are used.
    :param insecure_skip_tlsverify: InsecureSkipTLSVerify disables TLS certificate \
        verification when communicating with this server. This is strongly \
        discouraged.  You should use the CABundle instead.
    :param service: Service is a reference to the service for this API server.  It must \
        communicate on port 443 If the Service is nil, that means the handling for the \
        API groupversion is handled locally on this server. The call will simply \
        delegate to the normal handler chain to be fulfilled.
    """

    def __init__(
        self,
        group: str,
        group_priority_minimum: int,
        version: str,
        version_priority: int,
        ca_bundle: Optional[str] = None,
        insecure_skip_tlsverify: Optional[bool] = None,
        service: Optional[ServiceReference] = None,
    ):
        self.group = group
        self.groupPriorityMinimum = group_priority_minimum
        self.version = version
        self.versionPriority = version_priority
        self.caBundle = ca_bundle
        self.insecureSkipTLSVerify = insecure_skip_tlsverify
        self.service = service


class APIService(ApiRegistration):
    """
    :param metadata: None
    :param spec: Spec contains information for locating and communicating with a server
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ObjectMeta,
        spec: APIServiceSpec,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec



# flake8: noqa

from avionix.chart import (
    ChartBuilder,
    ChartDependency,
    ChartInfo,
    ChartMaintainer,
    Value,
    Values,
)
from avionix.kube.meta import ObjectMeta

from ._version import get_versions

__version__ = get_versions()["version"]
del get_versions



import re
from subprocess import CalledProcessError
from typing import Any, List, Optional, Tuple

from avionix._process_utils import custom_check_output


def _space_split(output_line: str):
    return [
        value
        for value in re.split(r"(\t|  +)", output_line)
        if not re.match(r"^\s*$", value)
    ]


def _get_name_locations(names: List[str], name_string: str):
    locs: List[Any] = []
    last_pos = 0
    for name in names:
        last_pos = name_string.find(name, last_pos)
        locs.append(last_pos)
    for i, loc in enumerate(locs):
        if i + 1 < len(locs):
            locs[i] = (loc, locs[i + 1])
            continue
        locs[i] = (loc, len(name_string))
    return locs


def _split_using_locations(locations: List[Tuple[int, int]], values_string: str):
    vals = []
    for i, loc in enumerate(locations):
        start = loc[0]
        end = loc[1]
        if i == len(locations) - 1:
            vals.append(values_string[start:].strip())
            continue
        vals.append(values_string[start:end].strip())
    return vals


def parse_output_to_dict(output: str):
    output_lines = output.split("\n")
    names = _space_split(output_lines[0])
    value_locations = _get_name_locations(names, output_lines[0])
    value_rows = []
    for line in output_lines[1:]:
        if line.strip():
            values = _split_using_locations(value_locations, line)
            value_rows.append(values)
    return {names[i]: row for i, row in enumerate(zip(*value_rows))}


class KubectlGetException(Exception):
    def __init__(self, msg: str):
        super().__init__(msg)


def kubectl_get(resource: str, namespace: Optional[str] = None, wide: bool = False):
    try:
        command = f"kubectl get {resource}"
        if namespace:
            command += f" -n {namespace}"
        if wide:
            command += " -o wide"
        return parse_output_to_dict(custom_check_output(command))
    except CalledProcessError as err:
        raise KubectlGetException(err.output)



from typing import Optional

from avionix.kube.base_objects import HelmYaml


class ChartMaintainer(HelmYaml):
    """
    :param name: The maintainers name (required for each maintainer)
    :param email: The maintainers email (optional for each maintainer)
    :param url: A URL for the maintainer (optional for each maintainer)
    """

    def __init__(
        self, name: str, email: Optional[str] = None, url: Optional[str] = None
    ):
        self.name = name
        self.email = email
        self.url = url



"""
Contains classes related to Role Based Access Control
"""

from typing import List, Optional

from avionix.kube.base_objects import RbacAuthorization
from avionix.kube.meta import LabelSelector, ObjectMeta
from avionix.yaml.yaml_handling import HelmYaml


class PolicyRule(HelmYaml):
    """
    :param api_groups: APIGroups is the name of the APIGroup that contains the \
        resources.  If multiple API groups are specified, any action requested against \
        one of the enumerated resources in any API group will be allowed.
    :param resources: Resources is a list of resources this rule applies to.  \
        ResourceAll represents all resources.
    :param verbs: Verbs is a list of Verbs that apply to ALL the ResourceKinds and \
        AttributeRestrictions contained in this rule.  VerbAll represents all kinds.
    :param resource_names: ResourceNames is an optional white list of names that the \
        rule applies to.  An empty set means that everything is allowed.
    :param non_resource_urls: NonResourceURLs is a set of partial urls that a user \
        should have access to.  '*'s are allowed, but only as the full, final step in \
        the path Since non-resource URLs are not namespaced, this field is only \
        applicable for ClusterRoles referenced from a ClusterRoleBinding. Rules can \
        either apply to API resources (such as "pods" or "secrets") or non-resource \
        URL paths (such as "/api"),  but not both.
    """

    def __init__(
        self,
        api_groups: List[str],
        resources: List[str],
        verbs: List[str],
        resource_names: Optional[List[str]] = None,
        non_resource_urls: Optional[List[str]] = None,
    ):
        self.apiGroups = api_groups
        self.nonResourceURLs = non_resource_urls
        self.resourceNames = resource_names
        self.resources = resources
        self.verbs = verbs


class Role(RbacAuthorization):
    """
    :param metadata: Standard object's metadata.
    :param rules: Rules holds all the PolicyRules for this Role
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ObjectMeta,
        rules: List[PolicyRule],
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.rules = rules


class RoleRef(HelmYaml):
    """
    :param name: Name is the name of resource being referenced
    :param api_group: APIGroup is the group for the resource being referenced
    :param kind: Kind is the type of resource being referenced
    """

    def __init__(self, name: str, api_group: str, kind: str):
        self.name = name
        self.apiGroup = api_group
        self.kind = kind


class Subject(HelmYaml):
    """
    :param name: Name of the object being referenced.
    :param kind: Kind of object being referenced. Values defined by this API group are \
        "User", "Group", and "ServiceAccount". If the Authorizer does not recognized \
        the kind value, the Authorizer should report an error.
    :param api_group: APIGroup holds the API group of the referenced subject. Defaults \
        to "" for ServiceAccount subjects. Defaults to "rbac.authorization.k8s.io" for \
        User and Group subjects.
    :param namespace: Namespace of the referenced object.  If the object kind is \
        non-namespace, such as "User" or "Group", and this value is not empty the \
        Authorizer should report an error.
    """

    def __init__(
        self,
        name: str,
        kind: str,
        api_group: Optional[str] = None,
        namespace: Optional[str] = None,
    ):
        self.name = name
        self.kind = kind
        self.apiGroup = api_group
        self.namespace = namespace


class RoleBinding(RbacAuthorization):
    """
    :param metadata: Standard object's metadata.
    :param role_ref: RoleRef can reference a Role in the current namespace or a \
        ClusterRole in the global namespace. If the RoleRef cannot be resolved, the \
        Authorizer must return an error.
    :param subjects: Subjects holds references to the objects the role applies to.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ObjectMeta,
        role_ref: RoleRef,
        subjects: Optional[List[Subject]] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.roleRef = role_ref
        self.subjects = subjects


class AggregationRule(HelmYaml):
    """
    :param cluster_role_selectors: ClusterRoleSelectors holds a list of selectors which \
        will be used to find ClusterRoles and create the rules. If any of the \
        selectors match, then the ClusterRole's permissions will be added
    """

    def __init__(self, cluster_role_selectors: List[LabelSelector]):
        self.clusterRoleSelectors = cluster_role_selectors


class ClusterRole(RbacAuthorization):
    """
    :param metadata: Standard object's metadata.
    :param aggregation_rule: AggregationRule is an optional field that describes how to \
        build the Rules for this ClusterRole. If AggregationRule is set, then the \
        Rules are controller managed and direct changes to Rules will be stomped by \
        the controller.
    :param rules: Rules holds all the PolicyRules for this ClusterRole
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ObjectMeta,
        aggregation_rule: Optional[AggregationRule] = None,
        rules: Optional[List[PolicyRule]] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.aggregationRule = aggregation_rule
        self.rules = rules


class ClusterRoleBinding(RbacAuthorization):
    """
    :param metadata: Standard object's metadata.
    :param role_ref: RoleRef can only reference a ClusterRole in the global namespace. \
        If the RoleRef cannot be resolved, the Authorizer must return an error.
    :param subjects: Subjects holds references to the objects the role applies to.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ObjectMeta,
        role_ref: RoleRef,
        subjects: Optional[List[Subject]] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.roleRef = role_ref
        self.subjects = subjects



from distutils.core import setup
import os
from pathlib import Path

from setuptools import find_packages

import versioneer

CODE_DIRECTORY = Path(__file__).parent


def read_file(filename):
    """Source the contents of a file"""
    with open(
        os.path.join(os.path.dirname(__file__), filename), encoding="utf-8"
    ) as file:
        return file.read()


setup(
    name="avionix",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    packages=find_packages(),
    long_description="Coming soon...",
    maintainer="Zach Brookler",
    maintainer_email="zachb1996@yahoo.com",
    description="A package for soldifying kubernetes structure and development by "
    "using objects and code rather than yaml",
    python_requires=">=3.6.1",
    install_requires=["pyyaml==5.3.1"],
    project_urls={
        "Source Code": "https://github.com/zbrookle/avionix",
        "Documentation": "https://github.com/zbrookle/avionix",
        "Bug Tracker": "https://github.com/zbrookle/avionix/issues",
    },
    url="https://avionix.readthedocs.io/en/latest/index.html",
    download_url="https://github.com/zbrookle/avionix/archive/master.zip",
    keywords=["kubernetes", "helm", "yaml", "docker", "infrastructure", "devops"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Typing :: Typed",
        "Operating System :: OS Independent",
    ],
    long_description_content_type="text/markdown",
    include_package_data=True,
)



import re


def post_uninstall_handle_error(message):
    return HelmError(message)


class ErrorFactory:
    def __init__(self, message: str):
        self._message = message

    def get_error(self):
        if re.match(r"Error: Kubernetes cluster unreachable.*", self._message):
            return ClusterUnavailableError(self._message)
        if re.match(r"Error: cannot re-use a name that is still in use", self._message):
            return ChartAlreadyInstalledError(self._message)
        if re.match(
            r"Error:.*unable to create new content in namespace \w+ because "
            r"it is being terminated",
            self._message,
        ):
            return NamespaceBeingTerminatedError(self._message)
        if re.match(".*namespaces.*not found.*", self._message):
            return NamespaceDoesNotExist(self._message)
        return None


class AvionixError(Exception):
    def __init__(self, msg: str):
        super().__init__(msg)


class ClusterUnavailableError(AvionixError):
    pass


class ChartAlreadyInstalledError(AvionixError):
    pass


class ChartNotInstalledError(AvionixError):
    pass


class HelmError(AvionixError):
    pass


class NamespaceDoesNotExist(AvionixError):
    def __init__(self, msg: str):
        super().__init__(
            f"{msg}To create namespace on installation, "
            f"use create_namespace=True in the ChartBuilder class"
        )


class NamespaceBeingTerminatedError(AvionixError):
    pass



from logging import info
import os
from pathlib import Path
import shutil
import time
from typing import Callable, Dict, Optional, Tuple

from avionix.chart import ChartBuilder
from avionix.errors import ChartAlreadyInstalledError
from avionix.testing import kubectl_get


class ChartInstallationContext:
    """
    Class to help with installing and uninstalling charts for testing
    """

    chart_id = 0

    def __init__(
        self,
        chart_builder: ChartBuilder,
        status_resource: str = "pods",
        timeout: int = 20,
        expected_status: Optional[set] = None,
        status_field: str = "STATUS",
        uninstall_func: Optional[Callable] = None,
        extra_installation_args: Optional[Dict[str, str]] = None,
        parallel: bool = False,
    ):
        self.chart_builder = chart_builder
        if parallel:
            self.chart_builder.namespace = f"test-{self.chart_id}"
            self.chart_id += 1
        self.status_resource = status_resource
        self.timeout = timeout
        if expected_status is None:
            self.expected_status = {"Running", "Success"}
        else:
            self.expected_status = expected_status
        self.__temp_dir = Path.cwd() / "tmp"
        self.__status_field = status_field
        self.__uninstall_func = uninstall_func
        self.extra_installation_args: Dict[str, str] = (
            {} if extra_installation_args is None else {}
        )

    def get_status_resources(self) -> Dict[str, Tuple[str]]:
        resources = kubectl_get(self.status_resource)
        if self.__status_field in resources:
            return resources[self.__status_field]
        return {}

    def wait_for_uninstall(self):
        while True:
            resources = self.get_status_resources()
            if not resources:
                break
            time.sleep(1)

    def __enter__(self):
        os.makedirs(str(self.__temp_dir), exist_ok=True)
        options = {"dependency-update": None, "wait": None, "create-namespace": ""}
        options.update(self.extra_installation_args)
        try:
            self.chart_builder.install_chart(options=options)
        except ChartAlreadyInstalledError:
            info("Chart already installed, uninstalling...")
            self.chart_builder.uninstall_chart()
            self.chart_builder.install_chart(options=options)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.__uninstall_func is not None:
            self.__uninstall_func()
        else:
            self.chart_builder.uninstall_chart()
            self.wait_for_uninstall()
        shutil.rmtree(self.__temp_dir)
        if os.path.exists(str(self.__temp_dir)):
            raise Exception("Should not exist")



"""
Classes related to metadata
"""

from datetime import datetime
from typing import List, Optional

from avionix.kube.base_objects import KubernetesBaseObject, Meta
from avionix.yaml.yaml_handling import HelmYaml


class APIResource(Meta):
    """
    :param name: name is the plural name of the resource.
    :param categories: categories is a list of the grouped resources this resource \
        belongs to (e.g. 'all')
    :param group: group is the preferred group of the resource.  Empty implies the \
        group of the containing resource list. For subresources, this may have a \
        different value, for example: Scale".
    :param namespaced: namespaced indicates if a resource is namespaced or not.
    :param short_names: shortNames is a list of suggested short names of the resource.
    :param singular_name: singularName is the singular name of the resource.  This \
        allows clients to handle plural and singular opaquely. The singularName is \
        more correct for reporting status on a single item and both singular and \
        plural are allowed from the kubectl CLI interface.
    :param storage_version_hash: The hash value of the storage version, the version \
        this resource is converted to when written to the data store. Value must be \
        treated as opaque by clients. Only equality comparison on the value is valid. \
        This is an alpha feature and may change or be removed in the future. The field \
        is populated by the apiserver only if the StorageVersionHash feature gate is \
        enabled. This field will remain optional even if it graduates.
    :param verbs: verbs is a list of supported kube verbs (this includes get, list, \
        watch, create, update, patch, delete, deletecollection, and proxy)
    :param version: version is the preferred version of the resource.  Empty implies \
        the version of the containing resource list For subresources, this may have a \
        different value, for example: v1 (while inside a v1beta1 version of the core \
        resource's group)".
    """

    def __init__(
        self,
        name: str,
        categories: List[str],
        group: str,
        namespaced: bool,
        short_names: List[str],
        singular_name: str,
        storage_version_hash: str,
        verbs: List[str],
        version: str,
    ):
        super().__init__()
        self.name = name
        self.categories = categories
        self.group = group
        self.namespaced = namespaced
        self.shortNames = short_names
        self.singularName = singular_name
        self.storageVersionHash = storage_version_hash
        self.verbs = verbs
        self.version = version

    # def to_dict(self):
    #     dictionary = super().to_dict()
    #     del dictionary["apiVersion"]
    #     return dictionary


class ManagedFieldsEntry(HelmYaml):
    """
    :param fields_type: FieldsType is the discriminator for the different fields format \
        and version. There is currently only one possible value: "FieldsV1"
    :param fields_v1: FieldsV1 holds the first JSON version format as described in the \
        "FieldsV1" type.
    :param manager: Manager is an identifier of the workflow managing these fields.
    :param operation: Operation is the type of operation which lead to this \
        ManagedFieldsEntry being created. The only valid values for this field are \
        'Apply' and 'Update'.
    :param time: Time is timestamp of when these fields were set. It should always be \
        empty if Operation is 'Apply'
    :param api_version: APIVersion defines the version of this resource that this field \
        set applies to. The format is "group/version" just like the top-level \
        APIVersion field. It is necessary to track the version of a field set because \
        it cannot be automatically converted.
    """

    def __init__(
        self,
        fields_type: Optional[str] = None,
        fields_v1: Optional[dict] = None,
        manager: Optional[str] = None,
        operation: Optional[str] = None,
        time: Optional[datetime] = None,
        api_version: Optional[str] = None,
    ):
        self.fieldsType = fields_type
        self.fieldsV1 = fields_v1
        self.manager = manager
        self.operation = operation
        self.time = self._get_kube_date_string(time)
        self.apiVersion = api_version

    @staticmethod
    def _get_kube_date_string(datetime_obj: Optional[datetime]):
        return (
            datetime_obj.strftime("%Y-%m-%dT%H:%M:%SZ%Z")
            if datetime_obj
            else datetime_obj
        )


class OwnerReference(KubernetesBaseObject):
    """
    :param name: Name of the referent. More info: \
        http://kubernetes.io/docs/user-guide/identifiers#names
    :param uid: UID of the referent. More info: \
        http://kubernetes.io/docs/user-guide/identifiers#uids
    :param controller: If true, this reference points to the managing controller.
    :param block_owner_deletion: If true, AND if the owner has the "foregroundDeletion" \
        finalizer, then the owner cannot be deleted from the key-value store until \
        this reference is removed. Defaults to false. To set this field, a user needs \
        "delete" permission of the owner, otherwise 422 (Unprocessable Entity) will be \
        returned.
    :param api_version: API version of the referent.
    """

    def __init__(
        self,
        name: str,
        uid: str,
        controller: Optional[bool] = None,
        block_owner_deletion: Optional[bool] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.name = name
        self.controller = controller
        self.blockOwnerDeletion = block_owner_deletion
        self.uid = uid


class ObjectMeta(HelmYaml):
    """
    :param annotations: Annotations is an unstructured key value map stored with a \
        resource that may be set by external tools to store and retrieve arbitrary \
        metadata. They are not queryable and should be preserved when modifying \
        objects. More info: http://kubernetes.io/docs/user-guide/annotations
    :param cluster_name: The name of the cluster which the object belongs to. This is \
        used to distinguish resources with same name and namespace in different \
        clusters. This field is not set anywhere right now and apiserver is going to \
        ignore it if set in create or update request.
    :param finalizers: Must be empty before the object is deleted from the registry. \
        Each entry is an identifier for the responsible component that will remove the \
        entry from the list. If the deletionTimestamp of the object is non-nil, \
        entries in this list can only be removed. Finalizers may be processed and \
        removed in any order.  Order is NOT enforced because it introduces significant \
        risk of stuck finalizers. finalizers is a shared field, any actor with \
        permission can reorder it. If the finalizer list is processed in order, then \
        this can lead to a situation in which the component responsible for the first \
        finalizer in the list is waiting for a signal (field value, external system, \
        or other) produced by a component responsible for a finalizer later in the \
        list, resulting in a deadlock. Without enforced ordering finalizers are free \
        to order amongst themselves and are not vulnerable to ordering changes in the \
        list.
    :param generate_name: GenerateName is an optional prefix, used by the server, to \
        generate a unique name ONLY IF the Name field has not been provided. If this \
        field is used, the name returned to the client will be different than the name \
        passed. This value will also be combined with a unique suffix. The provided \
        value has the same validation rules as the Name field, and may be truncated by \
        the length of the suffix required to make the value unique on the server.  If \
        this field is specified and the generated name exists, the server will NOT \
        return a 409 - instead, it will either return 201 Created or 500 with Reason \
        ServerTimeout indicating a unique name could not be found in the time \
        allotted, and the client should retry (optionally after the time indicated in \
        the Retry-After header).  Applied only if Name is not specified. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#idempotency  # noqa
    :param labels: Map of string keys and values that can be used to organize and \
        categorize (scope and select) objects. May match selectors of replication \
        controllers and services. More info: \
        http://kubernetes.io/docs/user-guide/labels
    :param managed_fields: ManagedFields maps workflow-id and version to the set of \
        fields that are managed by that workflow. This is mostly for internal \
        housekeeping, and users typically shouldn't need to set or understand this \
        field. A workflow can be the user's name, a controller's name, or the name of \
        a specific apply path like "ci-cd". The set of fields is always in the version \
        that the workflow used when modifying the object.
    :param name: Name must be unique within a namespace. Is required when creating \
        resources, although some resources may allow a client to request the \
        generation of an appropriate name automatically. Name is primarily intended \
        for creation idempotence and configuration definition. Cannot be updated. More \
        info: http://kubernetes.io/docs/user-guide/identifiers#names
    :param namespace: Namespace defines the space within each name must be unique. An \
        empty namespace is equivalent to the "default" namespace, but "default" is the \
        canonical representation. Not all objects are required to be scoped to a \
        namespace - the value of this field for those objects will be empty.  Must be \
        a DNS_LABEL. Cannot be updated. More info: \
        http://kubernetes.io/docs/user-guide/namespaces
    :param owner_references: List of objects depended by this object. If ALL objects in \
        the list have been deleted, this object will be garbage collected. If this \
        object is managed by a controller, then an entry in this list will point to \
        this controller, with the controller field set to true. There cannot be more \
        than one managing controller.
    """

    def __init__(
        self,
        annotations: Optional[dict] = None,
        cluster_name: Optional[str] = None,
        finalizers: Optional[List[str]] = None,
        generate_name: Optional[str] = None,
        labels: Optional[dict] = None,
        managed_fields: Optional[List[ManagedFieldsEntry]] = None,
        name: Optional[str] = None,
        namespace: Optional[str] = None,
        owner_references: Optional[List[OwnerReference]] = None,
    ):
        self.annotations = annotations
        self.clusterName = cluster_name
        self.finalizers = finalizers
        self.generateName = generate_name
        self.labels = labels
        self.managedFields = managed_fields
        self.name = name
        self.namespace = namespace
        self.ownerReferences = owner_references


class LabelSelectorRequirement(HelmYaml):
    """
    :param key: key is the label key that the selector applies to.
    :param operator: operator represents a key's relationship to a set of values. Valid \
        operators are In, NotIn, Exists and DoesNotExist.
    :param values: values is an array of string values. If the operator is In or NotIn, \
        the values array must be non-empty. If the operator is Exists or DoesNotExist, \
        the values array must be empty. This array is replaced during a strategic \
        merge patch.
    """

    def __init__(self, key: str, operator: str, values: Optional[List[str]] = None):
        self.key = key
        self.operator = operator
        self.values = values


class LabelSelector(HelmYaml):
    """
    :param match_labels: matchLabels is a map of {key,value} pairs. A single \
        {key,value} in the matchLabels map is equivalent to an element of \
        matchExpressions, whose key field is "key", the operator is "In", and the \
        values array contains only "value". The requirements are ANDed.
    :param match_expressions: matchExpressions is a list of label selector \
        requirements. The requirements are ANDed.
    """

    def __init__(
        self,
        match_labels: Optional[dict] = None,
        match_expressions: Optional[List[LabelSelectorRequirement]] = None,
    ):
        self.matchLabels = match_labels
        self.matchExpressions = match_expressions


class ListMeta(HelmYaml):
    """
    :param continue_: continue may be set if the user set a limit on the number of \
        items returned, and indicates that the server has more data available. The \
        value is opaque and may be used to issue another request to the endpoint that \
        served this list to retrieve the next set of available objects. Continuing a \
        consistent list may not be possible if the server configuration has changed or \
        more than a few minutes have passed. The resourceVersion field returned when \
        using this continue value will be identical to the value in the first \
        response, unless you have received this token from an error message.
    :param remaining_item_count: remainingItemCount is the number of subsequent items \
        in the list which are not included in this list response. If the list request \
        contained label or field selectors, then the number of remaining items is \
        unknown and the field will be left unset and omitted during serialization. If \
        the list is complete (either because it is not chunking or because this is the \
        last chunk), then there are no more remaining items and this field will be \
        left unset and omitted during serialization. Servers older than v1.15 do not \
        set this field. The intended use of the remainingItemCount is *estimating* the \
        size of a collection. Clients should not rely on the remainingItemCount to be \
        set or to be exact.
    """

    def __init__(self, continue_: str, remaining_item_count: int):
        self["continue"] = continue_
        self.remainingItemCount = remaining_item_count


class Patch(HelmYaml):
    """"""

    pass


class StatusCause(HelmYaml):
    """
    :param field: The field of the resource that has caused this error, as named by its \
        JSON serialization. May include dot and postfix notation for nested \
        attributes. Arrays are zero-indexed.  Fields may appear more than once in an \
        array of causes due to fields having multiple errors. Optional.  Examples:   \
        "name" - the field "name" on the current resource   "items[0].name" - the \
        field "name" on the first array entry in "items"
    :param message: A human-readable description of the cause of the error.  This field \
        may be presented as-is to a reader.
    :param reason: A machine-readable description of the cause of the error. If this \
        value is empty there is no information available.
    """

    def __init__(self, field: str, message: str, reason: str):
        self.field = field
        self.message = message
        self.reason = reason


class StatusDetails(Meta):
    """
    :param name: The name attribute of the resource associated with the status \
        StatusReason (when there is a single name which can be described).
    :param causes: The Causes array includes more details associated with the \
        StatusReason failure. Not all StatusReasons may provide detailed causes.
    :param group: The group attribute of the resource associated with the status \
        StatusReason.
    :param retry_after_seconds: If specified, the time in seconds before the operation \
        should be retried. Some errors may indicate the client must take an alternate \
        action - for those errors this field may indicate how long to wait before \
        taking the alternate action.
    :param uid: UID of the resource. (when there is a single resource which can be \
        described). More info: http://kubernetes.io/docs/user-guide/identifiers#uids
    """

    def __init__(
        self,
        name: str,
        causes: List[StatusCause],
        group: str,
        retry_after_seconds: Optional[int] = None,
        uid: Optional[str] = None,
    ):
        super().__init__()
        self.name = name
        self.causes = causes
        self.group = group
        self.retryAfterSeconds = retry_after_seconds
        self.uid = uid


class ServerAddressByClientCIDR(HelmYaml):
    """
    :param client_cidr: The CIDR with which clients can match their IP to figure out \
        the server address that they should use.
    :param server_address: Address of this server, suitable for a client that matches \
        the above CIDR. This can be a hostname, hostname:port, IP or IP:port.
    """

    def __init__(self, client_cidr: str, server_address: str):
        self.clientCIDR = client_cidr
        self.serverAddress = server_address


class GroupVersionForDiscovery(HelmYaml):
    """
    :param group_version: groupVersion specifies the API group and version in the form \
        "group/version"
    :param version: version specifies the version in the form of "version". This is to \
        save the clients the trouble of splitting the GroupVersion.
    """

    def __init__(self, group_version: str, version: str):
        self.groupVersion = group_version
        self.version = version


class APIGroup(Meta):
    """
    :param name: name is the name of the group.
    :param preferred_version: preferredVersion is the version preferred by the API \
        server, which probably is the storage version.
    :param server_address_by_client_cidrs: a map of client CIDR to server address that \
        is serving this group. This is to help clients reach servers in the most \
        network-efficient way possible. Clients can use the appropriate server address \
        as per the CIDR that they match. In case of multiple matches, clients should \
        use the longest matching CIDR. The server returns only those CIDRs that it \
        thinks that the client can match. For example: the master will return an \
        internal IP CIDR only, if the client reaches the server using an internal IP. \
        Server looks at X-Forwarded-For header or X-Real-Ip header or \
        request.RemoteAddr (in that order) to get the client IP.
    :param versions: versions are the versions supported in this group.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        name: str,
        preferred_version: GroupVersionForDiscovery,
        server_address_by_client_cidrs: List[ServerAddressByClientCIDR],
        versions: List[GroupVersionForDiscovery],
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.name = name
        self.preferredVersion = preferred_version
        self.serverAddressByClientCIDRs = server_address_by_client_cidrs
        self.versions = versions


class Preconditions(HelmYaml):
    """
    :param resource_version: Specifies the target ResourceVersion
    :param uid: Specifies the target UID.
    """

    def __init__(
        self, resource_version: Optional[str] = None, uid: Optional[str] = None
    ):
        self.resourceVersion = resource_version
        self.uid = uid


class DeleteOptions(KubernetesBaseObject):
    """
    :param dry_run: When present, indicates that modifications should not be persisted. \
        An invalid or unrecognized dryRun directive will result in an error response \
        and no further processing of the request. Valid values are: - All: all dry run \
        stages will be processed
    :param orphan_dependents: Deprecated: please use the PropagationPolicy, this field \
        will be deprecated in 1.7. Should the dependent objects be orphaned. If \
        true/false, the "orphan" finalizer will be added to/removed from the object's \
        finalizers list. Either this field or PropagationPolicy may be set, but not \
        both.
    :param preconditions: Must be fulfilled before a deletion is carried out. If not \
        possible, a 409 Conflict status will be returned.
    :param propagation_policy: Whether and how garbage collection will be performed. \
        Either this field or OrphanDependents may be set, but not both. The default \
        policy is decided by the existing finalizer set in the metadata.finalizers and \
        the resource-specific default policy. Acceptable values are: 'Orphan' - orphan \
        the dependents; 'Background' - allow the garbage collector to delete the \
        dependents in the background; 'Foreground' - a cascading policy that deletes \
        all dependents in the foreground.
    :param grace_period_seconds: The duration in seconds before the object should be \
        deleted. Value must be non-negative integer. The value zero indicates delete \
        immediately. If this value is nil, the default grace period for the specified \
        type will be used. Defaults to a per object value if not specified. zero means \
        delete immediately.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        dry_run: List[str],
        orphan_dependents: bool,
        preconditions: Preconditions,
        propagation_policy: str,
        grace_period_seconds: Optional[int] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.dryRun = dry_run
        self.orphanDependents = orphan_dependents
        self.preconditions = preconditions
        self.propagationPolicy = propagation_policy
        self.gracePeriodSeconds = grace_period_seconds


class APIVersions(KubernetesBaseObject):
    """
    :param server_address_by_client_cidrs: a map of client CIDR to server address that \
        is serving this group. This is to help clients reach servers in the most \
        network-efficient way possible. Clients can use the appropriate server address \
        as per the CIDR that they match. In case of multiple matches, clients should \
        use the longest matching CIDR. The server returns only those CIDRs that it \
        thinks that the client can match. For example: the master will return an \
        internal IP CIDR only, if the client reaches the server using an internal IP. \
        Server looks at X-Forwarded-For header or X-Real-Ip header or \
        request.RemoteAddr (in that order) to get the client IP.
    :param versions: versions are the api versions that are available.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        server_address_by_client_cidrs: List[ServerAddressByClientCIDR],
        versions: List[str],
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.serverAddressByClientCIDRs = server_address_by_client_cidrs
        self.versions = versions


class Status(KubernetesBaseObject):
    """
    :param metadata: Standard list metadata. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds  # noqa
    :param code: Suggested HTTP return code for this status, 0 if not set.
    :param message: A human-readable description of the status of this operation.
    :param reason: A machine-readable description of why this operation is in the \
        "Failure" status. If this value is empty there is no information available. A \
        Reason clarifies an HTTP status code but does not override it.
    :param details: Extended data associated with the reason.  Each reason may define \
        its own extended details. This field is optional and the data returned is not \
        guaranteed to conform to any schema except that defined by the reason type.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ListMeta,
        code: int,
        message: str,
        reason: str,
        details: Optional[StatusDetails] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.code = code
        self.message = message
        self.reason = reason
        self.details = details


class MicroTime(HelmYaml):
    """"""

    pass


class WatchEvent(HelmYaml):
    """
    :param object: Object is:

        - If Type is Added or Modified: the new state of the object.
        - If Type is Deleted: the state of the object immediately before deletion.
        - If Type is Error: Status is recommended; other types may make sense \
            depending on context.
    :param type: None
    """

    def __init__(self, object: str, type: str):
        self.object = object
        self.type = type


class Time(HelmYaml):
    """"""

    pass



"""
Contains RuntimeClass class
"""

from typing import List, Optional

from avionix.kube.base_objects import Node
from avionix.kube.core import Toleration
from avionix.kube.meta import ObjectMeta
from avionix.yaml.yaml_handling import HelmYaml


class Overhead(HelmYaml):
    """
    :param pod_fixed: PodFixed represents the fixed resource overhead associated with \
        running a pod.
    """

    def __init__(self, pod_fixed: dict):
        self.podFixed = pod_fixed


class Scheduling(HelmYaml):
    """
    :param tolerations: tolerations are appended (excluding duplicates) to pods running \
        with this RuntimeClass during admission, effectively unioning the set of nodes \
        tolerated by the pod and the RuntimeClass.
    :param node_selector: nodeSelector lists labels that must be present on nodes that \
        support this RuntimeClass. Pods using this RuntimeClass can only be scheduled \
        to a node matched by this selector. The RuntimeClass nodeSelector is merged \
        with a pod's existing nodeSelector. Any conflicts will cause the pod to be \
        rejected in admission.
    """

    def __init__(
        self, tolerations: List[Toleration], node_selector: Optional[dict] = None
    ):
        self.tolerations = tolerations
        self.nodeSelector = node_selector


class RuntimeClass(Node):
    """
    :param metadata: More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata  # noqa
    :param handler: Handler specifies the underlying runtime and configuration that the \
        CRI implementation will use to handle pods of this class. The possible values \
        are specific to the node & CRI configuration.  It is assumed that all handlers \
        are available on every node, and handlers of the same name are equivalent on \
        every node. For example, a handler called "runc" might specify that the runc \
        OCI runtime (using native Linux containers) will be used to run the containers \
        in a pod. The Handler must conform to the DNS Label (RFC 1123) requirements, \
        and is immutable.
    :param scheduling: Scheduling holds the scheduling constraints to ensure that pods \
        running with this RuntimeClass are scheduled to nodes that support it. If \
        scheduling is nil, this RuntimeClass is assumed to be supported by all nodes.
    :param overhead: Overhead represents the resource overhead associated with running \
        a pod for a given RuntimeClass. For more details, see \
        https://git.k8s.io/enhancements/keps/sig-node/20190226-pod-overhead.md This \
        field is alpha-level as of Kubernetes v1.15, and is only honored by servers \
        that enable the PodOverhead feature.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    _non_standard_version = "v1beta1"

    def __init__(
        self,
        metadata: ObjectMeta,
        handler: str,
        scheduling: Optional[Scheduling] = None,
        overhead: Optional[Overhead] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.handler = handler
        self.scheduling = scheduling
        self.overhead = overhead



"""
Classes related to management of storage resources
"""

from typing import List, Optional

from avionix.kube.base_objects import Storage
from avionix.kube.core import PersistentVolumeSpec, TopologySelectorTerm
from avionix.kube.meta import ObjectMeta
from avionix.yaml.yaml_handling import HelmYaml


class VolumeNodeResources(HelmYaml):
    """
    :param count: Maximum number of unique volumes managed by the CSI driver that can \
        be used on a node. A volume that is both attached and mounted on a node is \
        considered to be used once, not twice. The same rule applies for a unique \
        volume that is shared among multiple pods on the same node. If this field is \
        not specified, then the supported number of volumes on this node is unbounded.
    """

    def __init__(self, count: int):
        self.count = count


class CSINodeDriver(HelmYaml):
    """
    :param name: This is the name of the CSI driver that this object refers to. This \
        MUST be the same name returned by the CSI GetPluginName() call for that \
        driver.
    :param node_id: nodeID of the node from the driver point of view. This field \
        enables Kubernetes to communicate with storage systems that do not share the \
        same nomenclature for nodes. For example, Kubernetes may refer to a given node \
        as "node1", but the storage system may refer to the same node as "nodeA". When \
        Kubernetes issues a command to the storage system to attach a volume to a \
        specific node, it can use this field to refer to the node name using the ID \
        that the storage system will understand, e.g. "nodeA" instead of "node1". This \
        field is required.
    :param allocatable: allocatable represents the volume resources of a node that are \
        available for scheduling. This field is beta.
    :param topology_keys: topologyKeys is the list of keys supported by the driver. \
        When a driver is initialized on a cluster, it provides a set of topology keys \
        that it understands (e.g. "company.com/zone", "company.com/region"). When a \
        driver is initialized on a node, it provides the same topology keys along with \
        values. Kubelet will expose these topology keys as labels on its own node \
        object. When Kubernetes does topology aware provisioning, it can use this list \
        to determine which labels it should retrieve from the node object and pass \
        back to the driver. It is possible for different nodes to use different \
        topology keys. This can be empty if driver does not support topology.
    """

    def __init__(
        self,
        name: str,
        node_id: str,
        allocatable: Optional[VolumeNodeResources] = None,
        topology_keys: Optional[List[str]] = None,
    ):
        self.name = name
        self.allocatable = allocatable
        self.nodeID = node_id
        self.topologyKeys = topology_keys


class CSINodeSpec(HelmYaml):
    """
    :param drivers: drivers is a list of information of all CSI Drivers existing on a \
        node. If all drivers in the list are uninstalled, this can become empty.
    """

    def __init__(self, drivers: List[CSINodeDriver]):
        self.drivers = drivers


class StorageClass(Storage):
    """
    :param metadata: Standard object's metadata. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata  # noqa
    :param provisioner: Provisioner indicates the type of the provisioner.
    :param allow_volume_expansion: AllowVolumeExpansion shows whether the storage class \
        allow volume expand
    :param allowed_topologies: Restrict the node topologies where volumes can be \
        dynamically provisioned. Each volume plugin defines its own supported topology \
        specifications. An empty TopologySelectorTerm list means there is no topology \
        restriction. This field is only honored by servers that enable the \
        VolumeScheduling feature.
    :param parameters: Parameters holds the parameters for the provisioner that should \
        create volumes of this storage class.
    :param volume_binding_mode: VolumeBindingMode indicates how PersistentVolumeClaims \
        should be provisioned and bound.  When unset, VolumeBindingImmediate is used. \
        This field is only honored by servers that enable the VolumeScheduling \
        feature.
    :param mount_options: Dynamically provisioned PersistentVolumes of this storage \
        class are created with these mountOptions, e.g. ["ro", "soft"]. Not validated \
        - mount of the PVs will simply fail if one is invalid.
    :param reclaim_policy: Dynamically provisioned PersistentVolumes of this storage \
        class are created with this reclaimPolicy. Defaults to Delete.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ObjectMeta,
        provisioner: str,
        allow_volume_expansion: Optional[bool] = None,
        allowed_topologies: Optional[List[TopologySelectorTerm]] = None,
        parameters: Optional[dict] = None,
        volume_binding_mode: Optional[str] = None,
        mount_options: Optional[List[str]] = None,
        reclaim_policy: Optional[str] = None,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.allowVolumeExpansion = allow_volume_expansion
        self.allowedTopologies = allowed_topologies
        self.parameters = parameters
        self.provisioner = provisioner
        self.volumeBindingMode = volume_binding_mode
        self.mountOptions = mount_options
        self.reclaimPolicy = reclaim_policy


class VolumeAttachmentSource(HelmYaml):
    """
    :param inline_volume_spec: inlineVolumeSpec contains all the information necessary \
        to attach a persistent volume defined by a pod's inline VolumeSource. This \
        field is populated only for the CSIMigration feature. It contains translated \
        fields from a pod's inline VolumeSource to a PersistentVolumeSpec. This field \
        is alpha-level and is only honored by servers that enabled the CSIMigration \
        feature.
    :param persistent_volume_name: Name of the persistent volume to attach.
    """

    def __init__(
        self,
        inline_volume_spec: Optional[PersistentVolumeSpec] = None,
        persistent_volume_name: Optional[str] = None,
    ):
        self.inlineVolumeSpec = inline_volume_spec
        self.persistentVolumeName = persistent_volume_name


class VolumeAttachmentSpec(HelmYaml):
    """
    :param attacher: Attacher indicates the name of the volume driver that MUST handle \
        this request. This is the name returned by GetPluginName().
    :param source: Source represents the volume that should be attached.
    :param node_name: The node that the volume should be attached to.
    """

    def __init__(
        self, attacher: str, source: VolumeAttachmentSource, node_name: str,
    ):
        self.attacher = attacher
        self.source = source
        self.nodeName = node_name


class VolumeAttachment(Storage):
    """
    :param metadata: Standard object metadata. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata  # noqa
    :param spec: Specification of the desired attach/detach volume behavior. Populated \
        by the Kubernetes system.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ObjectMeta,
        spec: VolumeAttachmentSpec,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec


class CSIDriverSpec(HelmYaml):
    """
    :param attach_required: attachRequired indicates this CSI volume driver requires an \
        attach operation (because it implements the CSI ControllerPublishVolume() \
        method), and that the Kubernetes attach detach controller should call the \
        attach volume interface which checks the volumeattachment status and waits \
        until the volume is attached before proceeding to mounting. The CSI \
        external-attacher coordinates with CSI volume driver and updates the \
        volumeattachment status when the attach operation is complete. If the \
        CSIDriverRegistry feature gate is enabled and the value is specified to false, \
        the attach operation will be skipped. Otherwise the attach operation will be \
        called.
    :param volume_lifecycle_modes: volumeLifecycleModes defines what kind of volumes \
        this CSI volume driver supports. The default if the list is empty is \
        "Persistent", which is the usage defined by the CSI specification and \
        implemented in Kubernetes via the usual PV/PVC mechanism. The other mode is \
        "Ephemeral". In this mode, volumes are defined inline inside the pod spec with \
        CSIVolumeSource and their lifecycle is tied to the lifecycle of that pod. A \
        driver has to be aware of this because it is only going to get a \
        NodePublishVolume call for such a volume. For more information about \
        implementing this mode, see \
        https://kubernetes-csi.github.io/docs/ephemeral-local-volumes.html A driver \
        can support one or more of these modes and more modes may be added in the \
        future. This field is beta.
    :param pod_info_on_mount: If set to true, podInfoOnMount indicates this CSI volume \
        driver requires additional pod information (like podName, podUID, etc.) during \
        mount operations. If set to false, pod information will not be passed on \
        mount. Default is false. The CSI driver specifies podInfoOnMount as part of \
        driver deployment. If true, Kubelet will pass pod information as VolumeContext \
        in the CSI NodePublishVolume() calls. The CSI driver is responsible for \
        parsing and validating the information passed in as VolumeContext. The \
        following VolumeConext will be passed if podInfoOnMount is set to true. This \
        list might grow, but the prefix will be used. "csi.storage.k8s.io/pod.name": \
        pod.Name "csi.storage.k8s.io/pod.namespace": pod.Namespace \
        "csi.storage.k8s.io/pod.uid": string(pod.UID) "csi.storage.k8s.io/ephemeral": \
        "true" iff the volume is an ephemeral inline volume                            \
             defined by a CSIVolumeSource, otherwise "false"  \
        "csi.storage.k8s.io/ephemeral" is a new feature in Kubernetes 1.16. It is only \
        required for drivers which support both the "Persistent" and "Ephemeral" \
        VolumeLifecycleMode. Other drivers can leave pod info disabled and/or ignore \
        this field. As Kubernetes 1.15 doesn't support this field, drivers can only \
        support one mode when deployed on such a cluster and the deployment determines \
        which mode that is, for example via a command line parameter of the driver.
    """

    def __init__(
        self,
        attach_required: Optional[bool] = None,
        volume_lifecycle_modes: Optional[List[str]] = None,
        pod_info_on_mount: Optional[bool] = None,
    ):
        self.attachRequired = attach_required
        self.volumeLifecycleModes = volume_lifecycle_modes
        self.podInfoOnMount = pod_info_on_mount


class CSINode(Storage):
    """
    :param metadata: metadata.name must be the Kubernetes node name.
    :param spec: spec is the specification of CSINode
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self, metadata: ObjectMeta, spec: CSINodeSpec, api_version: Optional[str] = None
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec


class CSIDriver(Storage):
    """
    :param metadata: Standard object metadata. metadata.Name indicates the name of the \
        CSI driver that this object refers to; it MUST be the same name returned by \
        the CSI GetPluginName() call for that driver. The driver name must be 63 \
        characters or less, beginning and ending with an alphanumeric character \
        ([a-z0-9A-Z]) with dashes (-), dots (.), and alphanumerics between. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata  # noqa
    :param spec: Specification of the CSI Driver.
    :param api_version: APIVersion defines the versioned schema of this representation \
        of an object. Servers should convert recognized schemas to the latest internal \
        value, and may reject unrecognized values. More info: \
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa
    """

    def __init__(
        self,
        metadata: ObjectMeta,
        spec: CSIDriverSpec,
        api_version: Optional[str] = None,
    ):
        super().__init__(api_version)
        self.metadata = metadata
        self.spec = spec
